#ifndef TXmlDtdGenerator_h
#define TXmlDtdGenerator_h

#include "TXmlSetup.h"
#include "TObjArray.h"

class TCollection;
class TClass;
class TStreamerElement;
class TStreamerInfo;

class TXmlDtdGenerator : public TXmlSetup {
   public:
      TXmlDtdGenerator();
      TXmlDtdGenerator(const char* setup);
      TXmlDtdGenerator(const TXmlSetup& setup);
      virtual ~TXmlDtdGenerator();
 
      void Produce(const char* fname, TClass* onlyclass = 0);
      
      void AddClassSpace(TCollection* col = 0);
      
      void AddInstrumentedClass(TStreamerInfo* info);
      
      void AddBlackClass(TClass* cl);
      
      void AddUsedClass(TClass * cl);
 
   protected:  
      enum {dtd_none, dtd_attr, dtd_elem, dtd_charstar, dtd_base, dtd_object, dtd_objptr, 
            dtd_fixarray, dtd_array, dtd_fastobj1, dtd_fastobj2, 
            dtd_everyobj, dtd_stlp, dtd_objects, dtd_any};
       
      enum { MaxBaseTypeNum = 21};

      Int_t dtdType(TStreamerElement* el);
      
      const char* dtdBaseTypeName(int typ);
      const char* dtdUseBaseType(TStreamerElement* el);

      void ProduceDtdForItem(ofstream& fs, const char* name);
      
      void ProduceDtdForInstrumentedClass(ofstream& fs, TStreamerInfo* info);
      
      void ProduceDtdForBlackClass(ofstream& fs, TClass* cl);
      
      void ProduceObjectElement(ofstream& fs, const char* name, TClass* cl, Bool_t isPointer = kFALSE);
      
      void ProduceGeneralDtd(ofstream& fs, TClass* onlyclass = 0);
      void ProduceSpecificDtd(ofstream& fs, TClass* onlyclass = 0);
      
      TObjArray  fClassSpace;
      TObjArray  fInstrumentedClasses;
      TObjArray  fBlackClasses;
      
      TObjArray  fUsedClasses;

      TString fDtdBuf;
      Bool_t  fUsedBaseTypes[MaxBaseTypeNum];
      
   ClassDef(TXmlDtdGenerator,1);
};


#endif

