#ifndef TXmlEngine_h
#define TXmlEngine_h

#include "TObject.h"
#include "TString.h"

typedef void* xmlNodePointer;
typedef void* xmlNsPointer;
typedef void* xmlAttrPointer;
typedef void* xmlDocPointer;

class TXmlEngine : public TObject {
   protected:
      TXmlEngine();
      virtual ~TXmlEngine();
      static TXmlEngine* fInstance;
      
   public:

      static TXmlEngine* GetInstance() { return fInstance; }

      Bool_t HasProp(xmlNodePointer node, const char* name); 
        
      const char* GetProp(xmlNodePointer node, const char* name);

      xmlAttrPointer NewProp(xmlNodePointer node, 
                             xmlNsPointer ns,
                             const char* name, 
                             const char* value); 
      
      xmlNodePointer NewChild(xmlNodePointer parent, 
                              xmlNsPointer ns,
                              const char* name, 
                              const char* content = 0); 
                                  
      xmlNsPointer NewNS(xmlNodePointer node, 
                         const char* reference,
                         const char* name = 0);
                                  
      void AddChild(xmlNodePointer parent, xmlNodePointer child);
      
      void UnlinkChild(xmlNodePointer node);
      
      void FreeNode(xmlNodePointer node);
      
      const char* GetNodeName(xmlNodePointer node);
      
      const char* GetNodeContent(xmlNodePointer node);
      
      xmlNodePointer GetChild(xmlNodePointer node);
      
      xmlNodePointer GetParent(xmlNodePointer node);

      xmlNodePointer GetNext(xmlNodePointer node);
      
      void ShiftToNext(xmlNodePointer &node, Bool_t skipempty = kTRUE);
      
      void SkipEmpty(xmlNodePointer &node);
   
      xmlDocPointer NewDoc(const char* version = 0);   
      
      void AssignDtd(xmlDocPointer doc, const char* dtdname, const char* rootname);
      
      void FreeDoc(xmlDocPointer doc);
      
      void SaveDoc(xmlDocPointer doc, const char* filename, Int_t layout = 1);
      
      void DocSetRootElement(xmlDocPointer doc, xmlNodePointer node);
      
      xmlNodePointer DocGetRootElement(xmlDocPointer doc);
      
      xmlDocPointer ParseFile(const char* filename);
      
      Bool_t ValidateDocument(xmlDocPointer doc, Bool_t doout = kFALSE);
  
   protected: 
   
      TString  fStrBuf;
     
   ClassDef(TXmlEngine,1);  
};

extern TXmlEngine*  gXML;

#endif 
