#ifndef TXMLFILE_H
#define TXMLFILE_H

#include "TXmlEngine.h"
#include "TFile.h"
#include "TXmlSetup.h"

class TXmlKey;
class TList;
class TXmlDtdGenerator;
class TStreamerElement;
class TStreamerInfo;

class TXmlFile : public TFile, public TXmlSetup {
   protected:
       // Interface to basic system I/O routines
       virtual Int_t    SysOpen(const char*, Int_t, UInt_t) { return 0; }
       virtual Int_t    SysClose(Int_t) { return 0; }
       virtual Int_t    SysRead(Int_t, void*, Int_t) { return 0; }
       virtual Int_t    SysWrite(Int_t, const void*, Int_t) { return 0; }
       virtual Long64_t SysSeek(Int_t, Long64_t, Int_t) { return 0; }
       virtual Int_t    SysStat(Int_t, Long_t*, Long64_t*, Long_t*, Long_t*) { return 0; }
       virtual Int_t    SysSync(Int_t) { return 0; }
       
   private:    
      //let the compiler do the job. gcc complains when the following line is activated
      //TXmlFile(const TXmlFile &) {}            //Files cannot be copied
      void operator=(const TXmlFile &);

   public:
      TXmlFile();
      TXmlFile(const char* filename, Option_t* option = "read", const char* title = "title", Int_t compression = 1);
      virtual ~TXmlFile();

      virtual Bool_t    cd(const char* path = "");
      
      virtual void      Close(Option_t* = "") {} // *MENU*
      virtual void      Draw(Option_t* = "") {}
      virtual void      DrawMap(const char* ="*",Option_t* ="") {} // *MENU*
      virtual void      FillBuffer(char* &) {}
      virtual void      Flush() {}

      virtual Long64_t  GetEND() const { return 0; }
      virtual Int_t     GetErrno() const { return 0; }
      virtual void      ResetErrno() const {}

      virtual Int_t     GetNfree() const { return 0; }
      virtual Int_t     GetNbytesInfo() const {return 0; }
      virtual Int_t     GetNbytesFree() const {return 0; }
      virtual Long64_t  GetSeekFree() const {return 0; }
      virtual Long64_t  GetSeekInfo() const {return 0; }
      virtual Long64_t  GetSize() const { return 0; }

      virtual Bool_t    IsOpen() const;

      virtual void      MakeFree(Long64_t, Long64_t) {}
      virtual void      MakeProject(const char *, const char* ="*", Option_t* ="new") {} // *MENU*
      virtual void      Map() {} // *MENU*
      virtual void      Paint(Option_t* ="") {}
      virtual void      Print(Option_t* ="") const {}
      virtual Bool_t    ReadBuffer(char*, Int_t) { return kFALSE; }
      virtual void      ReadFree() {}
      virtual void      ReadStreamerInfo() {}
      virtual Int_t     Recover() { return 0; }
      virtual Int_t     ReOpen(Option_t *mode);
      virtual void      Seek(Long64_t, ERelativeTo=kBeg) {}

      virtual void      SetEND(Long64_t) {}
      virtual void      ShowStreamerInfo() {}
      virtual Int_t     Sizeof() const { return 0; }

      virtual void      UseCache(Int_t = 10, Int_t = TCache::kDfltPageSize) {}
      virtual Bool_t    WriteBuffer(const char*, Int_t) { return kFALSE; }
      virtual Int_t     Write(const char* =0, Int_t=0, Int_t=0) { return 0; }
      virtual void      WriteFree() {}
      virtual void      WriteHeader() {}
      virtual void      WriteStreamerInfo() {}

      // XML specific functions

      TXmlDtdGenerator* GetDtdGenerator() const { return fDtdGener; }
      
      void              Write(TObject* obj, const char* name = 0);
      void              Write(void* obj, TClass* objClass, const char* name);
      
      TObject*          Get(const char* name);
      void*             GetAny(const char* name);
      
//      static void       WriteOnly(const char* filename, TObject* obj, const char* opt = "2ooxoo");
//      static void       WriteOnly(const char* filename, const void* obj, TClass* objClass, const char* opt = "2ooxoo");
//      static void*      ReadOnly(const char* filename, const char* opt = "2ooxoo");

   protected:
      // functions to store streamer infos

      xmlNodePointer    CreateStreamerInfoNode();
      void              ReadStreamerInfos(xmlNodePointer fRootNode);
   
      void              StoreStreamerElement(xmlNodePointer node, TStreamerElement* elem);
      void              ReadStreamerElement(xmlNodePointer node, TStreamerInfo* info);
   

      Bool_t            ReadFromFile();
      
      void              SaveToFile();
      
      static void       ProduceFileNames(const char* filename, TString& fname, TString& dtdname);

      xmlDocPointer     fDoc;              //!
      
      TXmlDtdGenerator*  fDtdGener;        //!
      
   ClassDef(TXmlFile,1);
};



#endif

